// **** LED ring watch with ATtiny85, DS1302, and two buttons ****

// **** Required libraries ****
#include <Adafruit_NeoPixel.h>
#include <ThreeWire.h>  
#include <RtcDS1302.h>

// **** Configuration ****
#define LED_COUNT           16
#define PIN_LED              3 
#define PIN_RST              2  
#define PIN_DAT              1  
#define PIN_CLK              0 
#define PIN_BTN             A2 
#define LED_BRIGHTNESS     127 // Overall brightness of the LEDs.
#define LOOP_MAX            84 // Number of cycles for the "glow" effect.
#define PIXEL_SHIFT         13 // Number of pixels to be shifted to make the digits display match the LEDs.

// Create the pixels and RTC objects.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(LED_COUNT, PIN_LED, NEO_GRB + NEO_KHZ800);
ThreeWire myWire(PIN_DAT, PIN_CLK, PIN_RST);
RtcDS1302<ThreeWire> rtc(myWire);

// Global objects.
long count = 0; 
long deltaTime = 0;           // Delta time from tuning in milliseconds.
long numLoops = 0;            // Keeps track of the "acceleration" of the tuning buttons.
bool timeWasTuned = false;
RtcDateTime actTime;

// *** Initialize the clock.
void setup(){
  int k;
  // Prepare the buttons.
  pinMode(PIN_BTN, INPUT);
  // Prepare the LEDs and the RTC.
  pixels.begin();
  rtc.Begin();
  // The next line will "capture" the date and time when the sketch was compiled.
  RtcDateTime compileTime = RtcDateTime(__DATE__, __TIME__);
  // If the time of the real-time clock is "smaller" than this, it is likely that its battery is empty and it lost track of the actual time.
  // Hence, the next lines will make the RTC chip running, and if it reports a "smaller" time, the code will update the RTC with the compile time.
  if(rtc.GetIsWriteProtected()){
    rtc.SetIsWriteProtected(false);
  }
  if(!rtc.IsDateTimeValid()){
    rtc.SetDateTime(compileTime);
  }
  if(!rtc.GetIsRunning()){
    rtc.SetIsRunning(true);
  }
  RtcDateTime now = rtc.GetDateTime();
  if(now < compileTime){
    compileTime += 120; // compileTime is the time written to the binary code during the compilation of the code.
                        // The ATtiny must be transfered from the programmer to the clock circuit, and the circuit must be powerd on before the
                        // date and time can be transfered to the real-time-chip.
                        // Hence, we add two minutes for this transfering/handling process. 
    rtc.SetDateTime(compileTime);
    // Set all LEDs to blue for a second to show that the RTC has received a new time value.
    for(k = 0; k < 16; k++){
      pixels.setPixelColor(k, 0x00007F); 
    }
    pixels.show(); 
    delay(1000);
  }
  // Switch off all LEDs.
  for(k = 0; k < 16; k++){
    pixels.setPixelColor(k, 0x000000); 
  }
  pixels.show(); 
}

// *** Infinite loop: Check whether the time has changed and modify the LEDs accordingly.
void loop(){
  int buttonAnalogState;
  long tuningDirection;
  unsigned long color;
  unsigned long coeff;
  long seconds, minutes, hours;
  int k;

  // Check whether a button was pressed.
      // **** BEGIN: Original code from the alternative design using the DS1307 where the two buttons are connected to separate I/O pins. ****
      // **** tuningDirection = !digitalRead(PIN_BTN_UP) - !digitalRead(PIN_BTN_DN);
      // **** END:   Original code from the alternative design using the DS1307 where the two buttons are connected to separate I/O pins. ****
  // **** BEGIN: New code for using the DS1302 with two buttons both connected to one pin.
  buttonAnalogState = analogRead(PIN_BTN);
  if(buttonAnalogState < 100){
    // Value is around 0 --> no button is pressed.
    tuningDirection = 0;
  }
  else if (buttonAnalogState < 800){
    // Value is between 100 and 799 --> first button (with bigger resistor value) is pressed --> treat as "up" button.
    tuningDirection = 1;
  }
  else{
    // Value is greater than 799 --> second button (with smalller resistor value) is pressed --> treat as "down" button.
    tuningDirection = -1;
  }
  // **** END:   New code for using the DS1302 with two buttons both connected to one pin.
  if(tuningDirection != 0){
    deltaTime += tuningDirection * 100 * numLoops;
    numLoops++;
    timeWasTuned = true;
  }
  if((tuningDirection == 0) && timeWasTuned){ // Buttons were released.
    // Store the new time to the RTC.
    rtc.SetDateTime(actTime);
    numLoops = 0;
    deltaTime = 0;
    timeWasTuned = false;
  }
  actTime = rtc.GetDateTime();
  actTime += (deltaTime/1000);
  seconds = actTime.Second();
  minutes = actTime.Minute();
  hours   = actTime.Hour();
  for(k = 0; k < 6; k++){
    coeff = abs(((count + LOOP_MAX/6 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = (coeff << 16) + (40 << 8);
    if(seconds & (0x01 << k)){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  for(k = 6; k < 12; k++){
    coeff = abs(((count + LOOP_MAX/6 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = ((coeff*2 + 40) << 16) + (coeff/3);
    if(minutes & (0x01 << (k-6))){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  hours %= 12;
  if(hours == 0){
    hours = 12; // Midnight --> 0:00 = 12:00 a.m.; Noon --> 12:00 = 12:00 p.m.
  }
  for(k = 12; k < 16; k++){
    coeff = abs(((count + LOOP_MAX/4 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = (((unsigned long)(coeff*1.5)) << 8) + (coeff + 40);
    if(hours & (0x01 << (k-12))){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  pixels.show(); 
  delay(20);
  count = (count + 1)%LOOP_MAX;
}
